import { world, system, Player, Container, ItemStack } from '@minecraft/server';
import { ActionFormData } from '@minecraft/server-ui';

// --- CONFIGURACIÓN ---
const LANGUAGE = "en"; // Cambia a "en" para inglés o "es" para español

const CATCHER_ID = "multiversemc:net";
const MAX_MOBS = 10;
const MAX_HEALTH = 250;
const BLOCKED_MOBS = [
    'minecraft:armor_stand',
    'minecraft:ender_dragon',
    'minecraft:warden',
    'minecraft:wither',
    'minecraft:player'
];
const BLOCKED_BLOCKS = [
    // Marcos
    "minecraft:frame", "minecraft:glow_frame",

    // Cofres y variantes
    "minecraft:chest",
    "minecraft:trapped_chest",
    "minecraft:ender_chest",
    "minecraft:barrel",

    // Shulker boxes (todas las variantes de color)
    "minecraft:black_shulker_box",
    "minecraft:blue_shulker_box",
    "minecraft:brown_shulker_box",
    "minecraft:cyan_shulker_box",
    "minecraft:gray_shulker_box",
    "minecraft:green_shulker_box",
    "minecraft:light_blue_shulker_box",
    "minecraft:light_gray_shulker_box",
    "minecraft:lime_shulker_box",
    "minecraft:magenta_shulker_box",
    "minecraft:orange_shulker_box",
    "minecraft:pink_shulker_box",
    "minecraft:purple_shulker_box",
    "minecraft:red_shulker_box",
    "minecraft:undyed_shulker_box",
    "minecraft:white_shulker_box",
    "minecraft:yellow_shulker_box",

    // Hornos y variantes
    "minecraft:furnace",
    "minecraft:blast_furnace",
    "minecraft:smoker",

    // Mesas de trabajo y variantes
    "minecraft:crafting_table",
    "minecraft:cartography_table",
    "minecraft:smithing_table",
    "minecraft:loom",
    "minecraft:stonecutter",
    "minecraft:fletching_table",
    "minecraft:crafter",

    // Alquimia y encantamiento
    "minecraft:enchanting_table",
    "minecraft:brewing_stand",
    "minecraft:anvil",
    "minecraft:chipped_anvil",
    "minecraft:damaged_anvil",
    "minecraft:grindstone",

    // Compostador
    "minecraft:composter",

    // Dispensador y dropper
    "minecraft:dispenser",
    "minecraft:dropper",

    // Hoppers
    "minecraft:hopper",

    // Beehive y bee nest (también contienen items)
    "minecraft:beehive",
    "minecraft:bee_nest"
];


// --- TRADUCCIONES ---
const TRANSLATIONS = {
    en: {
        capture_fail_blocked: "You cannot capture that mob.",
        capture_fail_no_health: "This mob cannot be captured.",
        capture_fail_too_much_health: "That mob has too much health!",
        capture_fail_no_durability: "The catcher does not have enough durability!",
        capture_fail_full: "The catcher is full!",
        capture_success: "%s has been captured! (%s/%s)",
        release_fail_empty: "The catcher is empty!",
        release_success: "%s has been released!",
        lore_line1: "- %s/%s mobs",
        lore_line2: "Click on a mob to capture",
        lore_line3: "Click on a block to release",
        form_title: "[ Captured Mobs ]",
        form_body: "You have %s/%s mobs captured.",
        lang_select_title: "Select Language / Seleccionar idioma",
        lang_select_body: "Choose your language / Elige tu idioma",
        lang_option_en: "English",
        lang_option_es: "Español",
        lang_set_en: "Language set to English.",
        lang_set_es: "Idioma configurado a Español."
    },
    es: {
        capture_fail_blocked: "No puedes capturar ese mob.",
        capture_fail_no_health: "No se puede capturar este mob.",
        capture_fail_too_much_health: "¡Ese mob tiene demasiada vida!",
        capture_fail_no_durability: "¡La red no tiene suficiente durabilidad!",
        capture_fail_full: "¡La red está llena!",
        capture_success: "¡%s ha sido capturado! (%s/%s)",
        release_fail_empty: "¡La red está vacía!",
        release_success: "¡%s ha sido liberado!",
        lore_line1: "- %s/%s mobs",
        lore_line2: "Haz clic en un mob para capturar",
        lore_line3: "Haz clic en un bloque para liberar",
        form_title: "[ Mobs Capturados ]",
        form_body: "Tienes %s/%s mobs capturados.",
        lang_select_title: "Select Language / Seleccionar idioma",
        lang_select_body: "Choose your language / Elige tu idioma",
        lang_option_en: "English",
        lang_option_es: "Español",
        lang_set_en: "Language set to English.",
        lang_set_es: "Idioma configurado a Español."
    }
};

// --- IDIOMA POR JUGADOR ---
const playerLanguages = new Map(); // player.id -> "en" | "es"

function getPlayerLanguage(player) {
    return playerLanguages.get(player.id) || LANGUAGE;
}

function translateForPlayer(player, key, ...params) {
    const lang = getPlayerLanguage(player);
    let text = TRANSLATIONS[lang][key] || key;
    params.forEach(param => {
        text = text.replace(/%s/, param);
    });
    return text;
}

// --- FUNCIONES AUXILIARES ---
function toFriendlyName(str) {
    return str.replace(/_/g, ' ')
        .split(' ')
        .map(word => word.charAt(0).toUpperCase() + word.slice(1))
        .join(' ');
}
function getFriendlyName(entity) {
    const customName = entity.nameTag?.trim();
    if (customName && customName.length > 0) {
        return customName;
    }
    let id = entity.typeId || "unknown:entity";
    if (id.includes(':')) {
        id = id.split(':')[1];
    }
    return toFriendlyName(id);
}
function getNameFromCaptured(mob) {
    if (mob.customName && mob.customName.length > 0) {
        return mob.customName;
    }
    return toFriendlyName(mob.entityName || "unknown");
}
function getCatcherSlots(container) {
    const catchers = [];
    for (let i = 0; i < container.size; i++) {
        const item = container.getItem(i);
        if (item && item.typeId === CATCHER_ID) {
            catchers.push({ item, slot: i });
        }
    }
    return catchers;
}
function updateCatcherLore(item, player) {
    const captured = JSON.parse(item.getDynamicProperty('capturedEntities') || '[]');
    const lang = getPlayerLanguage(player);
    item.setLore([
        `§a${TRANSLATIONS[lang].lore_line1.replace("%s", captured.length).replace("%s", MAX_MOBS)}`,
        `§7${TRANSLATIONS[lang].lore_line2}`,
        `§7${TRANSLATIONS[lang].lore_line3}`
    ]);
}

// --- ACTUALIZA EL LORE CADA SEGUNDO ---
system.runInterval(() => {
    for (const player of world.getPlayers()) {
        const inv = player.getComponent("minecraft:inventory").container;
        for (const { item, slot } of getCatcherSlots(inv)) {
            updateCatcherLore(item, player);
            inv.setItem(slot, item);
        }
    }
}, 20);

// --- CAPTURAR MOB ---
world.beforeEvents.playerInteractWithEntity.subscribe(ev => {
    const { player, target: entity } = ev;
    system.run(() => {
        const inv = player.getComponent("minecraft:inventory").container;
        const item = inv.getItem(player.selectedSlotIndex);
        if (!item || item.typeId !== CATCHER_ID) return;

        if (BLOCKED_MOBS.includes(entity.typeId)) {
            player.sendMessage(translateForPlayer(player, "capture_fail_blocked"));
            return;
        }
        const health = entity.getComponent("minecraft:health");
        if (!health) {
            player.sendMessage(translateForPlayer(player, "capture_fail_no_health"));
            return;
        }
        if (health.currentValue > MAX_HEALTH) {
            player.sendMessage(translateForPlayer(player, "capture_fail_too_much_health"));
            return;
        }
        const durability = item.getComponent("durability");
        if (health.currentValue > (durability.maxDurability - durability.damage)) {
            player.sendMessage(translateForPlayer(player, "capture_fail_no_durability"));
            return;
        }
        const captured = JSON.parse(item.getDynamicProperty('capturedEntities') || '[]');
        if (captured.length >= MAX_MOBS) {
            player.sendMessage(translateForPlayer(player, "capture_fail_full"));
            return;
        }
        const structureName = `capture_${player.name}_${Date.now()}`;
        player.runCommand(`structure save ${structureName} ${entity.location.x} ${entity.location.y} ${entity.location.z} ${entity.location.x} ${entity.location.y} ${entity.location.z} true disk false`);
        const customName = entity.nameTag?.trim() || null;
        captured.push({
            structureName,
            entityType: entity.typeId,
            entityName: entity.typeId.split(":")[1],
            health: health.currentValue,
            customName
        });
        item.setDynamicProperty('capturedEntities', JSON.stringify(captured));
        durability.damage += health.currentValue;
        inv.setItem(player.selectedSlotIndex, item);

        const mobName = getFriendlyName(entity);
        entity.remove();

        player.sendMessage(translateForPlayer(player, "capture_success", mobName, captured.length.toString(), MAX_MOBS.toString()));
    });
});

// --- LIBERAR MOB Y FORMULARIO DE IDIOMA ---
const activeFormPlayers = new Set();

world.beforeEvents.playerInteractWithBlock.subscribe(ev => {
    const { block, player, itemStack } = ev;
    if (!itemStack || itemStack.typeId !== CATCHER_ID) return;

    // --- BLOQUES BLOQUEADOS ---
    if (BLOCKED_BLOCKS.includes(block.typeId)) return;

    // --- FORMULARIO DE IDIOMA ---
    if (player.isSneaking) {
        ev.cancel = true;
        if (activeFormPlayers.has(player.id)) return;
        activeFormPlayers.add(player.id);

        system.run(() => {
            const lang = getPlayerLanguage(player);
            const form = new ActionFormData()
                .title(TRANSLATIONS[lang].lang_select_title)
                .body(TRANSLATIONS[lang].lang_select_body)
                .button(TRANSLATIONS[lang].lang_option_en)
                .button(TRANSLATIONS[lang].lang_option_es);

            form.show(player).then(res => {
                activeFormPlayers.delete(player.id);
                if (res.canceled || res.selection === null) return;
                const selectedLang = res.selection === 0 ? "en" : "es";
                playerLanguages.set(player.id, selectedLang);
                player.sendMessage(TRANSLATIONS[selectedLang][selectedLang === "en" ? "lang_set_en" : "lang_set_es"]);
            }).catch(() => {
                activeFormPlayers.delete(player.id);
            });
        });
        return;
    }

    // --- LIBERAR MOB ---
    if ((player['cool_down'] ?? 0) > Date.now()) {
        ev.cancel = true;
        return;
    }
    player['cool_down'] = Date.now() + 1000;
    ev.cancel = true;

    system.run(() => {
        const captured = JSON.parse(itemStack.getDynamicProperty('capturedEntities') || '[]');
        if (captured.length === 0) {
            player.sendMessage(translateForPlayer(player, "release_fail_empty"));
            return;
        }
        const inv = player.getComponent("minecraft:inventory").container;
        if (captured.length === 1) {
            const mob = captured.shift();
            player.runCommand(`structure load ${mob.structureName} ${block.location.x} ${block.location.y + 1} ${block.location.z}`);
            player.runCommand(`structure delete ${mob.structureName}`);
            itemStack.setDynamicProperty('capturedEntities', JSON.stringify(captured));
            inv.setItem(player.selectedSlotIndex, itemStack);

            const mobName = getNameFromCaptured(mob);
            player.sendMessage(translateForPlayer(player, "release_success", mobName.toString()));
        } else {
            const form = new ActionFormData()
                .title(translateForPlayer(player, "form_title"))
                .body(translateForPlayer(player, "form_body", captured.length.toString(), MAX_MOBS.toString()));

            captured.forEach((mob, i) => {
                const name = getNameFromCaptured(mob);
                const isVanilla = mob.entityType.startsWith("minecraft:");
                const iconTexture = isVanilla ? "textures/ui/vanilla" : "textures/ui/addon";
                form.button(
                    `[ ${name} ]\n§7HP: ${Math.floor(mob.health)}`,
                    iconTexture
                );
            });

            form.show(player).then(res => {
                if (res.canceled || res.selection === null) return;
                const mob = captured[res.selection];
                player.runCommand(`structure load ${mob.structureName} ${block.location.x} ${block.location.y + 1} ${block.location.z}`);
                player.runCommand(`structure delete ${mob.structureName}`);
                captured.splice(res.selection, 1);
                itemStack.setDynamicProperty('capturedEntities', JSON.stringify(captured));
                inv.setItem(player.selectedSlotIndex, itemStack);
                const mobName = getNameFromCaptured(mob);
                player.sendMessage(translateForPlayer(player, "release_success", mobName.toString()));
            });
        }
    });
});
